#!/usr/bin/perl

###  perl 020_longestPepPicker.pl <multipleSeqFile> <addingSeqFile> <geneIDalp>

## 1st command
##   perl 020_longestPepPicker.pl 020_orthoFile.txt Homo_sapiens.GRCh37.69.pep.all.fa ENSG
## 2nd command
##   perl 020_longestPepPicker.pl 020_orthoFile.txt Pan_troglodytes.CHIMP2.1.4.69.pep.all.fa ENSPTRG
## 3rd command
##   perl 020_longestPepPicker.pl 020_orthoFile.txt Canis_familiaris.CanFam3.1.69.pep.all.fa ENSCAFG

use strict;
use warnings;

my $orthoFile = $ARGV[0];
my $seqDB     = $ARGV[1];
my $geneAlf   = $ARGV[2];

my $outDir    = "020_out_multiAAFiles";


### Make sure the options
unless ($geneAlf =~ /G$/){
  print "Make sure the thrid option, the alphabed part of gene ID.  End at G?  Program stopped.\n";
  exit;
}

##### Read the orthofile
my $refOrthoLines = &readFile($orthoFile);
my @orthoLines    = @$refOrthoLines;
#print "@orthoLines\n";


#### Read database
open(DB, "$seqDB") or die "Failed to open $seqDB\n";
my @dbLinesTemp = <DB>;
close DB;
my ($recNames, $recNamesLength, $recfasHash) = &fasHashReader(\@dbLinesTemp);
my @names = @$recNames;
my %nameLengthHash = %$recNamesLength;
my %fasHashDB = %$recfasHash;

### output dir
unless(-d "$outDir"){
  print "\n";
  print "$outDir was made.\n";
  system("mkdir $outDir");
}

my $count = 0;
##### Main program
if($geneAlf eq "ENSG"){
  print "Empty files will be made.\n";
  foreach my $orthoLine (@orthoLines) {
    chomp($orthoLine);
    #next unless $orthoLine =~ /ENSG\d+/;
    $count++;
    $orthoLine =~ /($geneAlf\d+)/;
    my $geneID = $1;
    print "$count. $geneID\n";
    my $fileName = $geneID.".txt";
    my $longest = &logestSeqPicker($geneID);
    my $nameOfLongestSeq = $longest->[0];
    open(OUT,">$outDir/$fileName");
    print OUT "$nameOfLongestSeq\n";
    print OUT "$fasHashDB{$nameOfLongestSeq}\n";
    close OUT;
  }
  print "\n";
} 
else {
  foreach my $orthoLine (@orthoLines) {
    chomp($orthoLine);
    #next unless $orthoLine =~ /ENSG\d+/;
    $count++;

    if($orthoLine =~ /$geneAlf/){
      $orthoLine =~ /(ENSG\d+).*($geneAlf\d+)/;
      my $humanID = $1;
      my $geneID  = $2;
      print "$count.$humanID $geneID\n";
      #print "geneID $geneID\n";
      my $fileName = $humanID.".txt";
      my $longest = &logestSeqPicker($geneID);
      my $nameOfLongestSeq = $longest->[0];
      open(OUT,">>$outDir/$fileName");
      print OUT "$nameOfLongestSeq\n";
      print OUT "$fasHashDB{$nameOfLongestSeq}\n";
      close OUT;
    }
    else {
      $orthoLine =~ /(ENSG\d+)/;
      my $humanID = $1;
      print "$count.$humanID NONE\n";
    }
  }
  print "\n";
}



####################
####### subroutines

sub readFile {
  my ($fileSR) = @_;
  my @linesSR = ();
  
  open(ADD, "$fileSR") or die "Failed to open $fileSR. Stopped.\n\n";
  while(my $line = <ADD>){
    unless($line =~ /^Ensembl/){
      chomp($line);
      #print "$line\n";
      push(@linesSR, $line);
    }
  }
  close ADD;
  return \@linesSR;
}


###########
sub logestSeqPicker{
  my ($geneIDsr) = @_;
  if(my @tempHist = grep(/$geneIDsr/, @names)){
    my @eachRecHash = ( );  
    foreach my $eachRec (@tempHist) {
      chomp($eachRec);
      #print "$eachRec\n";
      push(@eachRecHash,[$eachRec,$nameLengthHash{$eachRec}]);
    }
    my @sorted = sort{$b->[1] <=> $a->[1]}@eachRecHash;
    #foreach my $s (@sorted) {
    #  print "$s->[1]";
    #  print "  $s->[0]\n";
    #}
    #print "\n";
    
    my $longestSR = $sorted[0];
    #print "$longestSR\n";
    #exit;
    return $longestSR;
  } else {
    print "Not found $geneIDsr in DB. Stopped.\n";
    exit;
  }
}


###########
sub fasHashReader
{
  my $name = "";
  my $seq = "";
  my $seqLength = "";
  my $stone = 0;
  my @namesSR = ( );
  my %namesLengthSR = ( );  
  my %fasHashSR = ( );

  my ($inFileContSR) = @_;

  foreach my $inFileLine (@$inFileContSR) {
    chomp($inFileLine);
    if($inFileLine =~ /^>/){  
      if($stone == 0){$name = $inFileLine;
        $stone = 1;  
      } else {
        $seqLength = length($seq);

        push(@namesSR, $name);
        $fasHashSR{$name} = $seq;
        $namesLengthSR{$name} = $seqLength;

        $name = "$inFileLine";
        $seq = "";
      }
    } else {
      $seq .= $inFileLine; 
    }
  }
  $seqLength = length($seq);
  push(@namesSR, $name);
  $fasHashSR{$name} = $seq;
  $namesLengthSR{$name} = $seqLength;

  # ハッシュと配列をメインプログラムに返す．
  return (\@namesSR, \%namesLengthSR, \%fasHashSR);

}