#!/usr/bin/perl

###  perl 030_cDNApicker.pl <aaFileDir> <cDNAdatabase> <geneIDalp>

### Exampel
##  perl 030_cDNApicker.pl 020_out_multiAAFiles Homo_sapiens.GRCh37.69.cdna.all.fa ENSG
##  perl 030_cDNApicker.pl 020_out_multiAAFiles Pan_troglodytes.CHIMP2.1.4.69.cdna.all.fa ENSPTRG
##  perl 030_cDNApicker.pl 020_out_multiAAFiles Canis_familiaris.CanFam3.1.69.cdna.all.fa ENSCAFG


use strict;
use warnings;

my $aaFileDir = $ARGV[0];
my $seqDB     = $ARGV[1];
my $geneAlf   = $ARGV[2];
my $outDir    = "030_out_multiCDNAFiles";



### Make sure the options
unless ($geneAlf =~ /G$/){
  print "Make sure the thrid option, the alphabed part of gene ID.  End at G?  Program stopped.\n";
  exit;
}



#### Read database
open(DB, "$seqDB");
my @dbLinesTemp = <DB>;
close DB;
my ($recNames, $recNamesLength, $recfasHash) = &fasHashReader(\@dbLinesTemp);
my @cDNAnames = @$recNames;
my %cDNArecs = %$recfasHash;


#### read aa dir
opendir(DIR, "$aaFileDir") or die "Failed to open $aaFileDir. Stopped.\n";
my @aaDirs = readdir(DIR);
close DIR;


### output dir
unless(-d "$outDir"){
  print "\n";
  print "$outDir was made.\n";
  print "\n";
  system("mkdir $outDir");
  
  my $countFile = 0;
  print "Empty files will be made.\n";
  foreach my $cdnaFileName (@aaDirs) {
    next if $cdnaFileName =~ /^\./;
    $countFile++;
    print "$countFile. $cdnaFileName was made.\n";
    open(TEMPCDNA, ">$outDir/$cdnaFileName");
    close TEMPCDNA;
  }
  print "\n";
}




my $count = 0;
##### Main program
print "cDNA files will be added.\n";
foreach my $aaFileName (@aaDirs) {
  next if $aaFileName =~ /^\./;
  $count++;

  ## Pick up the name lines of AA file
  open(AAFILE,"$aaFileDir/$aaFileName");
  my @aaFile = <AAFILE>;
  close AAFILE;
  my ($refNamesSR, $refNamesLengthSR, $refFasHashSR) = &fasHashReader(\@aaFile);
  my @names = @$refNamesSR;

  ## Pick up the name line including adding species
  if(my $hitNameLine = join("",(grep/$geneAlf/,@names))){
    chomp($hitNameLine);
    $hitNameLine =~ /transcript:(\w+) /;
    my $cdnaID = $1;
    
    my $cDNAnameLine = join("",grep(/$cdnaID/,@cDNAnames));
    chomp($cDNAnameLine);
    
    print "$count. $aaFileName FOUND\n";
    open(OUT, ">>$outDir/$aaFileName");
    print OUT "$cDNAnameLine\n";
    print OUT "$cDNArecs{$cDNAnameLine}\n";
    close OUT;
  } else {
    print "$count. $aaFileName NOT FOUND\n";
  }
}
print "\n";



####################
####### subroutines


###########
sub fasHashReader
{
  my $name = "";
  my $seq = "";
  my $seqLength = "";
  my $stone = 0;
  my @namesSR = ( );
  my %namesLengthSR = ( );  
  my %fasHashSR = ( );

  my ($inFileContSR) = @_;

  foreach my $inFileLine (@$inFileContSR) {
    chomp($inFileLine);
    if($inFileLine =~ /^>/){  
      if($stone == 0){$name = $inFileLine;
        $stone = 1;  
      } else {
        $seqLength = length($seq);

        push(@namesSR, $name);
        $fasHashSR{$name} = $seq;
        $namesLengthSR{$name} = $seqLength;

        $name = "$inFileLine";
        $seq = "";
      }
    } else {
      $seq .= $inFileLine; 
    }
  }
  $seqLength = length($seq);
  push(@namesSR, $name);
  $fasHashSR{$name} = $seq;
  $namesLengthSR{$name} = $seqLength;

  return (\@namesSR, \%namesLengthSR, \%fasHashSR);

}